Attribute VB_Name = "Sort"
Option Explicit

Public Sub InsertSort(ByRef A() As Variant, ByVal Lb As Long, ByVal Ub As Long)
    Dim t As Variant
    Dim i As Long
    Dim j As Long

    ' sort A[Lb..Ub]
    For i = Lb + 1 To Ub
        t = A(i)

        ' shift elements down until insertion point found
        For j = i - 1 To Lb Step -1
            If A(j) <= t Then Exit For
            A(j + 1) = A(j)
        Next j

        ' insert
        A(j + 1) = t
    Next i
End Sub

Public Sub ShellSort(ByRef A() As Variant, ByVal Lb As Long, ByVal Ub As Long)
    Dim n As Long
    Dim i As Long
    Dim j As Long
    Dim s As Long
    Dim t As Long
    Dim ht As Long
    Dim tmp As Variant
    Static initialized As Boolean
    Static h(26) As Long
    
    If Not initialized Then
        initialized = True
        h(0) = 1
        h(1) = 5
        h(2) = 19
        h(3) = 41
        h(4) = 109
        h(5) = 209
        h(6) = 505
        h(7) = 929
        h(8) = 2161
        h(9) = 3905
        h(10) = 8929
        h(11) = 16001
        h(12) = 36289
        h(13) = 64769
        h(14) = 146305
        h(15) = 260609
        h(16) = 587521
        h(17) = 1045505
        h(18) = 2354689
        h(19) = 4188161
        h(20) = 9427969
        h(21) = 16764929
        h(22) = 37730305
        h(23) = 67084289
        h(24) = 150958081
        h(25) = 268386305
        h(26) = 603906049
    End If
    
    ' find t such that 3*h[t] >= n
    n = Ub - Lb + 1
    t = 0
    Do While 3 * h(t) < n
        t = t + 1
    Loop

    ' start with h[t-1] */
    If t > 0 Then t = t - 1

    ' sort array[lb..ub]
    Do While t >= 0
        ' sort by insertion in increments of h[t]
        ht = h(t)
        For i = Lb + ht To Ub
            tmp = A(i)
            For j = i - ht To Lb Step -ht
                If A(j) <= tmp Then Exit For
                A(j + ht) = A(j)
            Next j
            A(j + ht) = tmp
        Next i
        t = t - 1
    Loop
End Sub

Private Function Partition(ByRef A() As Variant, ByVal Lb As Long, ByVal Ub As Long) _
        As Long
    Dim t As Variant
    Dim pivot As Variant
    Dim i As Long
    Dim j As Long

    ' partition array[lb..ub]

    ' select pivot and exchange with 1st element
    pivot = A((Lb + Ub) \ 2)

    ' sort Lb+1 .. Ub based on pivot
    i = Lb - 1
    j = Ub + 1
    Do
        Do
            j = j - 1
        Loop While A(j) > pivot
        Do
            i = i + 1
        Loop While A(i) < pivot
        
        If i >= j Then Exit Do

        ' swap A(i), A(j)
        t = A(i)
        A(i) = A(j)
        A(j) = t
    Loop
    
    Partition = j
End Function

Public Sub QuickSort(ByRef A() As Variant, ByVal Lb As Long, ByVal Ub As Long)
    Dim m As Long

    ' sort array A(lb..ub)

    Do While Lb < Ub
        ' quickly sort short lists
        If (Ub - Lb <= 50) Then
            Call InsertSort(A, Lb, Ub)
            Exit Sub
        End If

        m = Partition(A, Lb, Ub)

        ' eliminate tail recursion and
        ' sort the smallest partition first
        ' to minimize stack requirements
        If m - Lb <= Ub - m Then
            Call QuickSort(A, Lb, m)
            Lb = m + 1
        Else
            Call QuickSort(A, m + 1, Ub)
            Ub = m
        End If
    Loop
End Sub

Public Sub QSort(ByRef A() As Variant, ByVal Lb As Long, ByVal Ub As Long)
    Dim lbStack(32) As Long
    Dim ubStack(32) As Long
    Dim sp As Long              ' stack pointer
    Dim lbx As Long             ' current lower-bound
    Dim ubx As Long             ' current upper-bound
    Dim m As Long
    Dim p As Long               ' index to pivot
    Dim i As Long
    Dim j As Long
    Dim t As Variant            ' temp used for exchanges

    lbStack(0) = Lb
    ubStack(0) = Ub
    sp = 0
    Do While sp >= 0
        lbx = lbStack(sp)
        ubx = ubStack(sp)

        Do While (lbx < ubx)

            ' select pivot and exchange with 1st element
            p = lbx + (ubx - lbx) \ 2

            ' exchange lbx, p
            t = A(lbx)
            A(lbx) = A(p)
            A(p) = t

            ' partition into two segments
            i = lbx + 1
            j = ubx
            Do
                Do While i < j
                    If A(lbx) <= A(i) Then Exit Do
                    i = i + 1
                Loop

                Do While j >= i
                    If A(j) <= A(lbx) Then Exit Do
                    j = j - 1
                Loop

                If i >= j Then Exit Do

                ' exchange i, j
                t = A(i)
                A(i) = A(j)
                A(j) = t

                j = j - 1
                i = i + 1
            Loop

            ' pivot belongs in A[j]
            ' exchange lbx, j
            t = A(lbx)
            A(lbx) = A(j)
            A(j) = t

            m = j

            ' keep processing smallest segment, and stack largest
            If m - lbx <= ubx - m Then
                If m + 1 < ubx Then
                    lbStack(sp) = m + 1
                    ubStack(sp) = ubx
                    sp = sp + 1
                End If
                ubx = m - 1
            Else
                If m - 1 > lbx Then
                    lbStack(sp) = lbx
                    ubStack(sp) = m - 1
                    sp = sp + 1
                End If
                lbx = m + 1
            End If
        Loop
        sp = sp - 1
    Loop
End Sub


